@@ -1,7 +1,7 @@
 /* lock.c :  functions for manipulating filesystem locks.
  *
  * ====================================================================
- * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -16,7 +16,6 @@
  */
 
 
-#include <assert.h>
 #include "svn_pools.h"
 #include "svn_error.h"
 #include "svn_path.h"
@@ -24,12 +23,10 @@
 #include "svn_hash.h"
 #include "svn_time.h"
 #include "svn_utf.h"
-#include "svn_md5.h"
 
 #include <apr_uuid.h>
 #include <apr_file_io.h>
 #include <apr_file_info.h>
-#include <apr_md5.h>
 
 #include "lock.h"
 #include "tree.h"
@@ -63,10 +60,11 @@ static const char *
 make_digest(const char *str,
             apr_pool_t *pool)
 {
-  unsigned char digest[APR_MD5_DIGESTSIZE];
+  svn_checksum_t *checksum;
 
-  apr_md5(digest, str, strlen(str));
-  return svn_md5_digest_to_cstring_display(digest, pool);
+  svn_checksum(&checksum, svn_checksum_md5, str, strlen(str), pool);
+
+  return svn_checksum_to_cstring_display(checksum, pool);
 }
 
 
@@ -147,17 +145,16 @@ write_digest_file(apr_hash_t *children,
                   apr_pool_t *pool)
 {
   svn_error_t *err = SVN_NO_ERROR;
-  apr_file_t *fd;
+  svn_stream_t *stream;
   apr_hash_index_t *hi;
   apr_hash_t *hash = apr_hash_make(pool);
   const char *tmp_path;
+  const char *rev_0_path;
 
-  SVN_ERR(svn_fs_fs__ensure_dir_exists(svn_path_join(fs->path, PATH_LOCKS_DIR, 
+  SVN_ERR(svn_fs_fs__ensure_dir_exists(svn_path_join(fs->path, PATH_LOCKS_DIR,
                                                      pool), fs, pool));
   SVN_ERR(svn_fs_fs__ensure_dir_exists(svn_path_dirname(digest_path, pool), fs,
                                        pool));
-  SVN_ERR(svn_io_open_unique_file2
-          (&fd, &tmp_path, digest_path, ".tmp", svn_io_file_del_none, pool));
 
   if (lock)
     {
@@ -196,23 +193,22 @@ write_digest_file(apr_hash_t *children,
                  children_list->data, children_list->len, pool);
     }
 
-  if ((err = svn_hash_write2(hash,
-                             svn_stream_from_aprfile(fd, pool),
-                             SVN_HASH_TERMINATOR, pool)))
+  SVN_ERR(svn_stream_open_unique(&stream, &tmp_path,
+                                 svn_path_dirname(digest_path, pool),
+                                 svn_io_file_del_none, pool, pool));
+  if ((err = svn_hash_write2(hash, stream, SVN_HASH_TERMINATOR, pool)))
     {
-      svn_error_clear(svn_io_file_close(fd, pool));
+      svn_error_clear(svn_stream_close(stream));
       return svn_error_createf(err->apr_err,
                                err,
                                _("Cannot write lock/entries hashfile '%s'"),
                                svn_path_local_style(tmp_path, pool));
     }
 
-  SVN_ERR(svn_io_file_close(fd, pool));
+  SVN_ERR(svn_stream_close(stream));
   SVN_ERR(svn_io_file_rename(tmp_path, digest_path, pool));
-  SVN_ERR(svn_fs_fs__dup_perms
-          (digest_path, svn_fs_fs__path_rev(fs, 0, pool), pool));
-
-  return SVN_NO_ERROR;
+  SVN_ERR(svn_fs_fs__path_rev_absolute(&rev_0_path, fs, 0, pool));
+  return svn_fs_fs__dup_perms(digest_path, rev_0_path, pool);
 }
 
 
@@ -230,7 +226,7 @@ read_digest_file(apr_hash_t **children_p,
   svn_error_t *err = SVN_NO_ERROR;
   svn_lock_t *lock;
   apr_hash_t *hash;
-  apr_file_t *fd;
+  svn_stream_t *stream;
   const char *val;
 
   if (lock_p)
@@ -238,7 +234,7 @@ read_digest_file(apr_hash_t **children_p,
   if (children_p)
     *children_p = apr_hash_make(pool);
 
-  err = svn_io_file_open(&fd, digest_path, APR_READ, APR_OS_DEFAULT, pool);
+  err = svn_stream_open_readonly(&stream, digest_path, pool, pool);
   if (err && APR_STATUS_IS_ENOENT(err->apr_err))
     {
       svn_error_clear(err);
@@ -249,20 +245,18 @@ read_digest_file(apr_hash_t **children_p,
   /* If our caller doesn't care about anything but the presence of the
      file... whatever. */
   if (! (lock_p || children_p))
-    return svn_io_file_close(fd, pool);
+    return svn_stream_close(stream);
 
   hash = apr_hash_make(pool);
-  if ((err = svn_hash_read2(hash,
-                            svn_stream_from_aprfile(fd, pool),
-                            SVN_HASH_TERMINATOR, pool)))
+  if ((err = svn_hash_read2(hash, stream, SVN_HASH_TERMINATOR, pool)))
     {
-      svn_error_clear(svn_io_file_close(fd, pool));
+      svn_error_clear(svn_stream_close(stream));
       return svn_error_createf(err->apr_err,
                                err,
                                _("Can't parse lock/entries hashfile '%s'"),
                                svn_path_local_style(digest_path, pool));
     }
-  SVN_ERR(svn_io_file_close(fd, pool));
+  SVN_ERR(svn_stream_close(stream));
 
   /* If our caller cares, see if we have a lock path in our hash. If
      so, we'll assume we have a lock here. */
@@ -283,7 +277,7 @@ read_digest_file(apr_hash_t **children_p,
 
       if (! ((val = hash_fetch(hash, IS_DAV_COMMENT_KEY, pool))))
         return svn_fs_fs__err_corrupt_lockfile(fs, path);
-      lock->is_dav_comment = (val[0] == '1') ? TRUE : FALSE;
+      lock->is_dav_comment = (val[0] == '1');
 
       if (! ((val = hash_fetch(hash, CREATION_DATE_KEY, pool))))
         return svn_fs_fs__err_corrupt_lockfile(fs, path);
@@ -326,10 +320,10 @@ set_lock(svn_fs_t *fs,
          apr_pool_t *pool)
 {
   svn_stringbuf_t *this_path = svn_stringbuf_create(lock->path, pool);
-  svn_stringbuf_t *last_child = svn_stringbuf_create("", pool);
+  const char *lock_digest_path = NULL;
   apr_pool_t *subpool;
 
-  assert(lock);
+  SVN_ERR_ASSERT(lock);
 
   /* Iterate in reverse, creating the lock for LOCK->path, and then
      just adding entries for its parent, until we reach a parent
@@ -357,15 +351,16 @@ set_lock(svn_fs_t *fs,
         {
           this_lock = lock;
           lock = NULL;
-          svn_stringbuf_set(last_child, digest_file);
+          lock_digest_path = apr_pstrdup(pool, digest_file);
         }
       else
         {
           /* If we already have an entry for this path, we're done. */
-          if (apr_hash_get(this_children, last_child->data, last_child->len))
+          if (apr_hash_get(this_children, lock_digest_path,
+                           APR_HASH_KEY_STRING))
             break;
-          apr_hash_set(this_children, last_child->data,
-                       last_child->len, (void *)1);
+          apr_hash_set(this_children, lock_digest_path,
+                       APR_HASH_KEY_STRING, (void *)1);
         }
       SVN_ERR(write_digest_file(this_children, this_lock, fs,
                                 digest_path, subpool));
@@ -388,13 +383,13 @@ delete_lock(svn_fs_t *fs,
             apr_pool_t *pool)
 {
   svn_stringbuf_t *this_path = svn_stringbuf_create(lock->path, pool);
-  svn_stringbuf_t *child_to_kill = svn_stringbuf_create("", pool);
+  const char *child_to_kill = NULL;
   apr_pool_t *subpool;
 
-  assert(lock);
+  SVN_ERR_ASSERT(lock);
 
   /* Iterate in reverse, deleting the lock for LOCK->path, and then
-     pruning entries from its parents. */
+     deleting its entry as it appears in each of its parents. */
   subpool = svn_pool_create(pool);
   while (1729)
     {
@@ -412,32 +407,27 @@ delete_lock(svn_fs_t *fs,
       SVN_ERR(read_digest_file(&this_children, &this_lock, fs,
                                digest_path, subpool));
 
-      /* If we are supposed to drop the last entry from this path's
-         children list, do so. */
-      if (child_to_kill->len)
-        apr_hash_set(this_children, child_to_kill->data,
-                     child_to_kill->len, NULL);
-
       /* Delete the lock (first time through only). */
       if (lock)
         {
           this_lock = NULL;
           lock = NULL;
+          child_to_kill = apr_pstrdup(pool, digest_file);
         }
 
+      if (child_to_kill)
+        apr_hash_set(this_children, child_to_kill, APR_HASH_KEY_STRING, NULL);
+
       if (! (this_lock || apr_hash_count(this_children) != 0))
         {
           /* Special case:  no goodz, no file.  And remember to nix
              the entry for it in its parent. */
-          svn_stringbuf_set(child_to_kill,
-                            svn_path_basename(digest_path, subpool));
           SVN_ERR(svn_io_remove_file(digest_path, subpool));
         }
       else
         {
           SVN_ERR(write_digest_file(this_children, this_lock, fs,
                                     digest_path, subpool));
-          svn_stringbuf_setempty(child_to_kill);
         }
 
       /* Prep for next iteration, or bail if we're done. */
@@ -807,9 +797,7 @@ unlock_body(void *baton, apr_pool_t *pool)
     }
 
   /* Remove lock and lock token files. */
-  SVN_ERR(delete_lock(ub->fs, lock, pool));
-
-  return SVN_NO_ERROR;
+  return delete_lock(ub->fs, lock, pool);
 }
 
 
@@ -843,9 +831,7 @@ svn_fs_fs__lock(svn_lock_t **lock_p,
   lb.steal_lock = steal_lock;
   lb.pool = pool;
 
-  SVN_ERR(svn_fs_fs__with_write_lock(fs, lock_body, &lb, pool));
-
-  return SVN_NO_ERROR;
+  return svn_fs_fs__with_write_lock(fs, lock_body, &lb, pool);
 }
 
 
@@ -883,9 +869,7 @@ svn_fs_fs__unlock(svn_fs_t *fs,
   ub.token = token;
   ub.break_lock = break_lock;
 
-  SVN_ERR(svn_fs_fs__with_write_lock(fs, unlock_body, &ub, pool));
-
-  return SVN_NO_ERROR;
+  return svn_fs_fs__with_write_lock(fs, unlock_body, &ub, pool);
 }
 
 
